@@ -23,6 +23,7 @@ pub struct Polling {
     client: Client,
     confirms: u64,
     end: Option<u64>,
+    forever: bool,
     latest: u64,
     live_blocks: BTreeMap<u64, BlockInfo>,
     ptr: u64,
@@ -30,7 +31,7 @@ pub struct Polling {
 
 impl Polling {
     /// new polling service
-    pub async fn new(ptr: u64, end: Option<u64>, env: Env) -> Result<Self> {
+    pub async fn new(end: Option<u64>, env: Env, forever: bool, ptr: u64) -> Result<Self> {
         let client = Client::new(env.endpoints, Duration::from_millis(env.timeout), env.retry)?;
         let batch = env.batch_blocks as usize;
 
@@ -40,6 +41,7 @@ impl Polling {
             confirms: env.confirms,
             client,
             end,
+            forever,
             latest: 0,
             live_blocks: Default::default(),
             ptr,
@@ -155,18 +157,17 @@ impl Polling {
                 blocks.drain(..);
             }
 
-            // # Safty
-            //
-            // this will never happen since we have an empty check above
-            let new_ptr = *polling.last().ok_or(Error::ParseBlockPtrFailed)?;
+            // poll blocks and dm logging
             let mut blocks = self.client.poll(polling.into_iter()).await?;
             self.cmp_live_blocks(&mut blocks)?;
             for b in blocks {
+                let cur = b.height;
                 Self::dm_log(b)?;
+                // # Safty
+                //
+                // only update ptr after dm_log
+                self.ptr = cur + 1;
             }
-
-            // update ptr
-            self.ptr = new_ptr + 1;
         }
 
         Ok(())
@@ -181,8 +182,23 @@ impl Polling {
         }
 
         loop {
-            self.latest = self.client.get_current_block().await?.height;
-            self.poll(self.ptr..=self.latest).await?;
+            let r: Result<()> = {
+                self.latest = self.client.get_current_block().await?.height;
+                self.poll(self.ptr..=self.latest).await?;
+                Ok(())
+            };
+
+            // restart when network error occurs
+            if let Err(e) = r {
+                log::error!("{:?}", e);
+
+                if self.forever {
+                    log::info!("restarting...");
+                    continue;
+                } else {
+                    return Err(e);
+                }
+            }
 
             // sleep and waiting for new blocks
             log::info!(
